local Sorting = PetJournalEnhanced:NewModule("Sorting","AceEvent-3.0")
local BreedInfo  = LibStub("LibPetBreedInfo-1.0")
local BREED_HIGH_CONFIDENCE = 2
local MAX_BALANCED = 1
local MAX_SPEED = 2
local MAX_STAMINA = 3
local MAX_ATTACK = 4

local ASCENDING =  1
local DESCENDING = 2
local USER = nil


 petMapping = {}

local SetPet, GetPet
do
	local petStore = {}
	function SetPet(pet)
		if pet.isOwned then
			petStore[pet.petID] = pet
		else
			petStore[pet.speciesID] = pet
		end
	end

	function Sorting:GetPet(petID)
		return petStore[petID]
	end
end

local sortFunctions
do 
	local function GetCompareFunc(var,func,direction)
		return function(a,b)
			local order = Sorting.db.global.sorting.order
			local avar = a[var] 
			local bvar = b[var]
			if type(avar) == "boolean" then 
				avar = avar and 1 or 0
				bvar = bvar and 1 or 0 
			end
			
			if avar == bvar and func then 
				return func(a,b)
			end
			
			if direction then
				if direction == ASCENDING then
					return avar < bvar
				elseif  direction == DESCENDING then
					return avar > bvar
				end
			elseif  order == ASCENDING then
				return avar < bvar
			elseif order == DESCENDING then
				--if not avar or not bvar then print(avar,bvar,var) print(a.name,b.name) end
				return avar > bvar
			end
		end
	end

	local function ComposeSortFunction(...)
		local args = {...}
		local func = nil
		for i=#args,1,-2 do
			local name = args[i-1];
			local direction = args[i]
			func = GetCompareFunc(name,func,direction)
		end
		return func
	end

	sortFunctions =  {
			ComposeSortFunction("isOwned",DESCENDING,"level",  USER,"name", ASCENDING),--level
			ComposeSortFunction("isOwned",DESCENDING,"name",   USER,"level",DESCENDING),--alpha
			ComposeSortFunction("isOwned",DESCENDING,"petType",USER,"level",DESCENDING,"name",ASCENDING),--pet type
			ComposeSortFunction("isOwned",DESCENDING,"rarity", USER,"level",DESCENDING,"name",ASCENDING),--rarity
			ComposeSortFunction("isOwned",DESCENDING,"maxStat",USER,"level",DESCENDING,"name",ASCENDING),--specialization
			ComposeSortFunction("isOwned",DESCENDING,"breed",USER,"level",DESCENDING,"name",ASCENDING),--specialization
		}
end

--Return highest stat for a pet. Check constants for return values.
local function GetMaxStat(maxHealth,attack,speed)
	
	local maxHealth = tonumber(maxHealth)
	local stamina = tonumber(math.floor((maxHealth -100)/5))
	local attack = tonumber(attack)
	local speed = tonumber(speed)
	
	if attack > speed and attack > stamina then 
		return MAX_ATTACK 
	elseif speed > attack and speed > stamina  then 
		return MAX_SPEED 
	elseif stamina > attack and stamina > speed  then 
		return MAX_STAMINA 
	end
	return MAX_BALANCED
end

function Sorting:OnInitialize()
	self.ZoneFiltering = PetJournalEnhanced:GetModule("ZoneFiltering")
	self.callbacks = LibStub("CallbackHandler-1.0"):New(self)
	
	local defaults = {
		global = {
			filtering = {
				rarity = {[1]=true,[2]=true,[3]=true,[4]=true},
				breed = {[3]=true,[4]=true,[5]=true,[6]=true,[7]=true,[8]=true,[9]=true,[10]=true,[11]=true,[12]=true,},
				specialization = {[1]=true,[2]=true,[3]=true,[4]=true},
				level = {[1]=true,[2]=true,[3]=true,[4]=true},
				quantity = {[1]=true,[2]=true,[3]=true},
				currentZone = false,
				canBattle =true,
				cantBattle = true,
				unknownZone = true,
				cantTrade = true,
				canTrade = true,
				abilityType = {[1]=true,[2]=true,[3]=true,[3]=true,[4]=true,[5]=true,[6]=true,[7]=true,[8]=true,[9]=true,[10]=true},
			},
			sorting = {
				selection = 1,
				order = ASCENDING,
				favoritesFirst = false,
			},
		}
	}
	local db = LibStub("AceDB-3.0"):New("PetJournalEnhancedDB", {} , true)
	local name = self:GetName()
	self.db = db:RegisterNamespace(name, defaults)
	self.db.global.filtering.unknownZone = true --always reset to true
	self.filtering = self.db.global.filtering
	self.sorting = self.db.global.sorting
	
	
	self:RegisterEvent("ZONE_CHANGED");
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("PET_JOURNAL_PET_DELETED")
	
	
	local LibPetJournal = LibStub("LibPetJournal-2.0")
	LibPetJournal.RegisterCallback(self,"PostPetListUpdated", "ScanPets")
	
	PetJournalSearchBox:HookScript("OnTextChanged",function()  Sorting:SortPets() end)
	hooksecurefunc(C_PetJournal,"SetFavorite",Sorting.C_PetJournal_SetFavorite)
end


function Sorting.C_PetJournal_SetFavorite()
	Sorting:UpdatePets()
end


function Sorting:GetPetByIndex(index)
	return petMapping[index]
end

function Sorting:GetNumPets()
	return #petMapping
end

function Sorting:Reset()
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_COLLECTED, true)
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_FAVORITES, false);
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_NOT_COLLECTED, true);
	C_PetJournal.AddAllPetTypesFilter();
	C_PetJournal.AddAllPetSourcesFilter();
	
	self.ZoneFiltering:SetAllFiltered(true)
	
	local filtering = self.db.global.filtering
	local sorting = self.db.global.sorting
	
	for i=1,#filtering.rarity do
		filtering.rarity[i] = true
	end
	
	for i=1,#filtering.abilityType do
		filtering.abilityType[i] = true
	end
	
	for i=1,#filtering.level do
		filtering.level[i] = true
	end
	
	for i=1,#filtering.specialization do
		filtering.specialization[i] = true
	end
	
	for i=1,#filtering.quantity do
		filtering.quantity[i] = true
	end
	
	for i=1,#filtering.breed do
		filtering.breed[i] = true
	end
	
	filtering.currentZone = false
	filtering.canBattle = true
	filtering.cantBattle = true
	filtering.unknownZone = true
	filtering.canTrade = true
	filtering.cantTrade = true

	sorting.selection = 2
	sorting.order = ASCENDING
	sorting.favoritesFirst = false
	
	self:SortPets()
end

function Sorting:ScanPets()
	self:SortPets()
end

function Sorting:ZONE_CHANGED_NEW_AREA()
	self:SortPets()
end

function Sorting:ZONE_CHANGED()
	self:SortPets()
end

function Sorting:PETJOURNAL_ENHANCED_OPTIONS_UPDATE()
	self:SortPets()
end

function Sorting:PET_JOURNAL_PET_DELETED()
	self:SortPets()
end

function Sorting:SortPets()
	local db = self.db.global
	local filtering = db.filtering
	local display = db.display
	local numPets = C_PetJournal.GetNumPets(PetJournal.isWild)
	local currentZoneId =  GetCurrentMapAreaID()
	local ZoneFiltering = self.ZoneFiltering
	
	wipe(petMapping)

	local abilityFilter = 0
	for i=1,#filtering.abilityType do
		if filtering.abilityType[i] then
			abilityFilter = bit.bor(abilityFilter,2^i);
		end
	end

	for i=1,numPets do
		local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle, tradable, unique = C_PetJournal.GetPetInfoByIndex(i, PetJournal.isWild)
		
		local health, maxHealth, attack, speed, rarity
		if petID then
			 health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID)
		end
		
		local id = nil
		if isOwned then id = petID else id = speciesID end 
		local pet = self:GetPet(id)
		
		if not pet then
			pet = {}
			pet.speciesID = speciesID
			pet.petID = petID 
			pet.isOwned = isOwned
			pet.name = name
			pet.petType = petType
			pet.canBattle = canBattle
			
			pet.rarity = rarity or 0				
			pet.maxStat = 0
			pet.abilityFilter = 0
			pet.breed = 0
			pet.breedConfidence = 0
			pet.level = level or 0
			
			if isOwned and canBattle and maxHealth and attack and speed then
				pet.maxStat = GetMaxStat(maxHealth,attack,speed)
				local breed, breedConfidence = BreedInfo:GetBreedByPetID(petID)
				if breed then
					pet.breed, pet.breedConfidence = breed, breedConfidence
				end
			end
			
			if canBattle then
				local abilityIDs = C_PetJournal.GetPetAbilityList(pet.speciesID);
				for j=1, #abilityIDs do
					local _,_,abilityType = C_PetJournal.GetPetAbilityInfo(abilityIDs[j]);
					pet.abilityFilter = bit.bor(pet.abilityFilter,2^abilityType);
				end
			else
				pet.level = 0
				pet.rarity = 0
				pet.petType = 0
			end
			SetPet(pet)
		end
		
		pet.owned = C_PetJournal.GetNumCollectedInfo(speciesID)
		pet.favorite = favorite
		pet.index = i
		
		
		if isOwned and canBattle and  pet.breedConfidence < BREED_HIGH_CONFIDENCE and level and level > pet.level then
			local breed, breedConfidence = BreedInfo:GetBreedByPetID(petID)
			if breed then
				pet.breed, pet.breedConfidence = breed, breedConfidence
			end
		end
		
		if canBattle then
			pet.level = level
		end
		
		local exclude = false
		local zones = ZoneFiltering:GetZonesBySpeciesID(speciesID)

		if not filtering.cantBattle and not canBattle  then
			exclude = true
		elseif not filtering.canBattle and canBattle  then
			exclude = true
		elseif not filtering.canTrade and tradable  then
			exclude = true	
		elseif not filtering.cantTrade and not tradable  then
			exclude = true	
		elseif pet.rarity and pet.rarity > 0 and not filtering.rarity[pet.rarity] then
			exclude = true
		elseif pet.owned > 0 and not filtering.quantity[pet.owned] then
			exclude = true
		elseif pet.maxStat > 0 and  not filtering.specialization[pet.maxStat] then
			exclude = true
		elseif filtering.currentZone and (zones == nil or not zones[currentZoneId]) then
			exclude = true
		elseif not filtering.unknownZone and not (string.find(sourceText,"Pet Battle:") or string.find(sourceText,"Zone:"))  then
			exclude = true
		elseif pet.breed > 0 and  not filtering.breed[pet.breed] then
			exclude =true
		elseif not filtering.level[1] and pet.level <= 10 then
			exclude = true
		elseif not filtering.level[2] and pet.level <= 20 and pet.level >= 11 then
			exclude = true
		elseif not filtering.level[3] and pet.level <= 24 and pet.level >= 20 then
			exclude = true
		elseif not filtering.level[4] and pet.level == 25 then
			exclude = true
		elseif canBattle and bit.band(abilityFilter,pet.abilityFilter) == 0 then
			exclude = true
		elseif zones then
			local filtered = false
			for k,v in pairs(zones) do
				filtered = filtered or ZoneFiltering:GetFiltered(k)
			end
			if filtered == false then exclude = true end
		end

		if not exclude then
			table.insert(petMapping,pet)
		end
	end
	
	local sortFunc = sortFunctions[db.sorting.selection]
	table.sort(petMapping,sortFunc)
	self.callbacks:Fire("PETS_SORTED")
end
Sorting.UpdatePets = Sorting.SortPets
